------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--              OCARINA.AADL.PARSER.COMPONENTS.SUBCOMPONENTS                --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--               Copyright (C) 2004-2006, GET-Telecom Paris.                --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

with Locations;
with Ocarina.Nodes;

with Ocarina.AADL.Lexer;
with Ocarina.AADL.Tokens;
with Ocarina.AADL.Parser.Components.Modes;
with Ocarina.AADL.Parser.Properties;
with Ocarina.AADL.Parser.Identifiers;
with Ocarina.Builder.Components.Subcomponents;

package body Ocarina.AADL.Parser.Components.Subcomponents is

   --------------------
   -- P_Subcomponent --
   --------------------

   --  subcomponent ::=
   --     defining_subcomponent_identifier :
   --     component_category [ component classifier_reference ]
   --     [ { { subcomponent_property_association
   --         | contained_property_association }+ } ]
   --     [ in_modes ] ;

   --  subcomponent_refinement ::=
   --     defining_subcomponent_identifier : refined to
   --     component_category [ component classifier_reference ]
   --     [ { { subcomponent_property_association
   --         | contained_property_association }+ } ]
   --     [ in_modes ] ;

   function P_Subcomponent
     (Container : Types.Node_Id;
      Refinable : Boolean)
     return Node_Id
   is
      use Locations;
      use Ocarina.Nodes;
      use Tokens;
      use Lexer;
      use Parser.Components.Modes;
      use Parser.Properties;
      use Parser.Identifiers;
      use Ocarina.Builder.Components.Subcomponents;

      Subcomponent    : Node_Id;          --  output
      Identifier      : Node_Id;
      Is_Refinement   : Boolean;
      Category        : Component_Category;
      Component_Ref   : Node_Id;
--      Component_Prop  : List_Id;
      Component_Modes : Node_Id;
      Code            : Parsing_Code;
      Loc             : Location;
      OK              : Boolean;

   begin
      P_Identifier_Refined_To (Refinable_To_RT (Refinable), False,
                               PC_Subcomponent, PC_Subcomponent_Refinement,
                               T_Semicolon, Identifier, Is_Refinement, OK);
      if not OK then
         return No_Node;
      end if;

      if Is_Refinement then
         Code := PC_Subcomponent_Refinement;
      else
         Code := PC_Subcomponent;
      end if;

      Scan_Token;
      Category := P_Component_Category;
      if Category = CC_Unknown then
         Skip_Tokens (T_Semicolon);
         return No_Node;
      end if;

      Save_Lexer (Loc);
      Scan_Token;
      if Token = T_Identifier then
         Restore_Lexer (Loc);
         Component_Ref := P_Entity_Reference (Code);

         if No (Component_Ref) then
            Skip_Tokens (T_Semicolon);
            return No_Node;
         end if;
      else
         Component_Ref := No_Node;
         Restore_Lexer (Loc);
      end if;

      Subcomponent := Add_New_Subcomponent
        (Loc => Ocarina.Nodes.Loc (Identifier),
         Name => Identifier,
         Comp_Impl => Container,
         Is_Refinement => Is_Refinement,
         Category => Category,
         In_Modes => No_Node);
--      P_Property_Associations (Subcomponent, True, PAT_Simple_Or_Contained,
--                               T_Semicolon, Code, Component_Prop, OK);
      OK := P_Property_Associations (Subcomponent, True,
                                     PAT_Simple_Or_Contained, Code);
      if not OK then
         return No_Node;
      end if;

      Save_Lexer (Loc);
      Scan_Token;
      if Token = T_In then
         Component_Modes := P_In_Modes (PC_In_Modes);

         if No (Component_Modes) then
            Skip_Tokens (T_Semicolon);
            return No_Node;
         end if;

         Set_In_Modes (Subcomponent, Component_Modes);
      else
         Restore_Lexer (Loc);
      end if;

      Save_Lexer (Loc);
      Scan_Token;
      if Token /= T_Semicolon then
         DPE (Code, T_Semicolon);
         Restore_Lexer (Loc);
         return No_Node;
      end if;

      if Subcomponent /= No_Node then
         Set_Entity_Ref (Subcomponent, Component_Ref);
      end if;

      return Subcomponent;
   end P_Subcomponent;

end Ocarina.AADL.Parser.Components.Subcomponents;
