------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--           O C A R I N A . A A D L . P A R S E R _ E R R O R S            --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--               Copyright (C) 2004-2006, GET-Telecom Paris.                --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

with Types;      use Types;

with Ocarina.AADL.Tokens; use Ocarina.AADL.Tokens;

package Ocarina.AADL.Parser_Errors is

   --  Parsing Codes

   type Parsing_Code is
     (PC_None_Statement,
      PC_Items_List,
      PC_Classifier_Reference,
      PC_Identifiers,
      PC_Defining_Identifier,
      PC_Defining_Name,

      PC_AADL_Specification,
      PC_AADL_Declaration,
      PC_Package_Specification,
      PC_Package_Declarations,
      PC_Annex_Library,
      PC_Annex_Subclause,

      PC_Component,
      PC_Component_Category,
      PC_Unique_Component_Type_Identifier,
      PC_Component_Type,
      PC_Component_Type_Extension,
      PC_Unique_Component_Impl_Name,
      PC_Component_Implementation,
      PC_Component_Implementation_Extension,

      PC_Connections,
      PC_Connection,
      PC_Connection_Refinement,
      PC_Data_Connection,
      PC_Data_Connection_Refinement,
      PC_Event_Connection,
      PC_Event_Connection_Refinement,
      PC_Event_Data_Connection,
      PC_Event_Data_Connection_Refinement,
      PC_Port_Group_Connection,
      PC_Port_Group_Connection_Refinement,
      PC_Parameter_Connection,
      PC_Parameter_Connection_Refinement,
      PC_Access_Connection,
      PC_Access_Connection_Refinement,

      PC_Features,
      PC_Refines_Type,
      PC_Feature,
      PC_Feature_Refinement,
      PC_Parameters,
      PC_Parameter,
      PC_Parameter_Refinement,

      PC_Flow_Identifier,
      PC_Subcomponent_Flow_Identifier,
      PC_Flow_Specifications,
      PC_Flow_Spec,
      PC_Flow_Spec_Refinement,
      PC_Flow_Source_Spec,
      PC_Flow_Source_Spec_Refinement,
      PC_Flow_Sink_Spec,
      PC_Flow_Sink_Spec_Refinement,
      PC_Flow_Path_Spec,
      PC_Flow_Path_Spec_Refinement,
      PC_End_To_End_Flow_Spec,
      PC_End_To_End_Flow_Refinement,

      PC_Flow_Implementations,
      PC_Flow_Implementation,
      PC_Flow_Implementation_Refinement,
      PC_Flow_Path_Implementation,
      PC_Flow_Path_Implementation_Refinement,
      PC_Flow_Sink_Implementation,
      PC_Flow_Sink_Implementation_Refinement,
      PC_Flow_Source_Implementation,
      PC_Flow_Source_Implementation_Refinement,

      PC_Mode,
      PC_Mode_Refinement,
      PC_In_Modes,
      PC_In_Modes_And_Transitions,
      PC_Mode_Or_Mode_Transition,
      PC_Mode_Or_Transition,
      PC_Mode_Transition,

      PC_Port_Type,
      PC_Port_Group_Spec,
      PC_Port_Group_Refinement,
      PC_Port_Group_Type,
      PC_Port_Group_Type_Extension,
      PC_Port_Spec,
      PC_Port_Refinement,
      PC_Port_Spec_Or_Port_Group_Spec,
      PC_Port_Refinement_Or_Port_Group_Refinement,
      PC_Unique_Port_Group_Type_Reference,
      PC_Unique_Port_Identifier,

      PC_Properties,
      PC_Property_Set,
      PC_Property_Declaration,

      PC_Property_Type_Declaration,
      PC_Property_Type_Designator,
      PC_Unique_Property_Type_Identifier,

      PC_Enumeration_Type,
      PC_Units_Type,
      PC_Unit_Definition,
      PC_Number_Type,
      PC_Range_Type,
      PC_Number_Range,
      PC_Real_Range,
      PC_Integer_Range,
      PC_Classifier_Type,
      PC_Reference_Type,
      PC_Referable_Element_Category,

      PC_Property_Name_Declaration,
      PC_Property_Owner_Category,
      PC_Single_Valued_Property,
      PC_Multi_Valued_Property,

      PC_Property_Constant,
      PC_Unique_Property_Constant_Identifier,
      PC_Single_Valued_Property_Constant,
      PC_Multi_Valued_Property_Constant,
      PC_Constant_Signed_Integer_Value,
      PC_Constant_Signed_Real_Value,
      PC_Constant_String_Value,
      PC_Constant_Property_Value,

      PC_Property_Association,
      PC_Access_Property_Association,
      PC_Contained_Property_Association,
      PC_Property_Association_Or_Contained_Property_Association,

      PC_Property_Expression,
      PC_Property_List_Value,
      PC_Property_Term,
      PC_Boolean_Property_Term,
      PC_Component_Classifier_Term,
      PC_Reference_Term,
      PC_Boolean_Term,
      PC_Number_Range_Term,
      PC_Integer_Range_Term,
      PC_Real_Range_Term,

      PC_Data_Subprogram_Spec,
      PC_Data_Subprogram_Refinement,
      PC_Server_Subprogram,
      PC_Server_Subprogram_Refinement,
      PC_Subprogram_Call,
      PC_Subprogram_Call_Sequences,
      PC_Subprogram_Call_Sequence,

      PC_Subcomponents,
      PC_Subcomponent,
      PC_Subcomponent_Refinement,
      PC_Subcomponent_Access,
      PC_Subcomponent_Access_Refinement,
      PC_Subcomponent_Access_Classifier);

   --  Error Message Code

   type Error_Message_Code is
     (EMC_Access_Property_Association_Is_Not_Allowed,
      EMC_Contained_Property_Association_Is_Not_Allowed,
      EMC_Debug,
      EMC_Invalid_Range,
      EMC_List_Is_Empty,
      EMC_No_Defining_Identifier,
      EMC_Refinement_Is_Not_Allowed,
      EMC_Extends_Incompatible_Entity);

   procedure Display_Parsing_Error
     (Code      : Parsing_Code;
      Error_Msg : Error_Message_Code);
   procedure DPE (Code : Parsing_Code; Error_Msg : Error_Message_Code)
     renames Display_Parsing_Error;
   --  Display an output message:
   --     Location: parsing ..., <Msg>

   procedure Display_Parsing_Error (Code : Parsing_Code);
   procedure DPE (Code : Parsing_Code) renames Display_Parsing_Error;
   --  Display an output error message:
   --     Location: parsing ..., unexpected `Current_Token_Image`

   procedure Display_Parsing_Error (Code       : Parsing_Code;
                                    Identifier : Name_Id);
   procedure DPE (Code       : Parsing_Code;
                  Identifier : Name_Id) renames Display_Parsing_Error;
   --  Display an output error message:
   --     Location: parsing ..., identifier <...> expected, found
   --     Current_Token_Image

   procedure Display_Parsing_Error (Code           : Parsing_Code;
                                    Expected_Token : Token_Type);
   procedure DPE (Code           : Parsing_Code;
                  Expected_Token : Token_Type) renames Display_Parsing_Error;
   --  Display an output error message:
   --     Location: parsing ..., token ... expected, found Current_Token_Image

   procedure Display_Parsing_Error (Code            : Parsing_Code;
                                    Expected_Tokens : Token_List_Type);
   procedure DPE (Code            : Parsing_Code;
                  Expected_Tokens : Token_List_Type)
     renames Display_Parsing_Error;
   --  Display an output error message:
   --     Location: parsing ..., token ... or ... or ... expected,
   --     found Current_Token_Image

   function Image (Code : Parsing_Code) return String;
   --  Return corresponding string of given parsing code

   function Image (Code : Error_Message_Code) return String;
   --  Return corresponding string of given error message code

end Ocarina.AADL.Parser_Errors;
