------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--            OCARINA.AADL.PRINTER.COMPONENTS.SUBPROGRAM_CALLS              --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--               Copyright (C) 2005-2007, GET-Telecom Paris.                --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

with Output;

with Ocarina.Nodes;
with Ocarina.Nutils;
with Ocarina.AADL.Printer.Components.Modes;
with Ocarina.AADL.Printer.Properties;
with Ocarina.AADL.Printer.Identifiers;

package body Ocarina.AADL.Printer.Components.Subprogram_Calls is

   use Output;
   use Ocarina.Nodes;
   use Ocarina.Nutils;
   use Ocarina.AADL.Printer.Components.Modes;
   use Ocarina.AADL.Printer.Properties;
   use Ocarina.AADL.Printer.Identifiers;

   ---------------------------
   -- Print_Subprogram_Call --
   ---------------------------

   procedure Print_Subprogram_Call
     (Node    : Node_Id;
      Options : Output_Options)
   is
   begin
      Print_Identifier (Identifier (Node));
      Write_Space;
      Print_Tokens ((T_Colon, T_Subprogram));
      Write_Space;
      Print_Entity_Reference (Entity_Ref (Node));
      Print_Contained_Property_Associations
        (Ocarina.Nodes.Properties (Node), Options);
      Print_Token (T_Semicolon);
   end Print_Subprogram_Call;

   ------------------------------------
   -- Print_Subprogram_Call_Sequence --
   ------------------------------------

   procedure Print_Subprogram_Call_Sequence
     (Node    : Node_Id;
      Options : Output_Options)
   is
      Ident     : constant Node_Id := Identifier (Node);
      In_Modes  : constant Node_Id := Ocarina.Nodes.In_Modes (Node);
      List_Node : Node_Id;
   begin
      if Present (Ident) then
         Write_Indentation;
         Print_Identifier (Ident);
         Write_Space;
         Print_Token (T_Colon);
         Write_Eol;
      end if;

      if not Is_Empty (Ocarina.Nodes.Subprogram_Calls (Node)) then
         List_Node := First_Node (Ocarina.Nodes.Subprogram_Calls (Node));

         while Present (List_Node) loop
            Write_Indentation;

            if List_Node = First_Node (Ocarina.Nodes.Subprogram_Calls
                                       (Node))
            then
               Print_Token (T_Left_Curly_Bracket);
            else
               Write_Space;
            end if;

            Print_Subprogram_Call (List_Node, Options);

            if List_Node = Last_Node
              (Ocarina.Nodes.Subprogram_Calls
               (Node))
            then
               Print_Token (T_Right_Curly_Bracket);
            end if;

            Write_Eol;
            List_Node := Next_Node (List_Node);
         end loop;
      end if;

      Print_In_Modes (In_Modes, Options);
      Print_Token (T_Semicolon);
      Write_Eol;
   end Print_Subprogram_Call_Sequence;

end Ocarina.AADL.Printer.Components.Subprogram_Calls;
