------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--            O C A R I N A . C H E C K E R _ T E S T S U I T E             --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--                 Copyright (C) 2007, GET-Telecom Paris.                   --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

with Ocarina.Checker;
with Types;

package Ocarina.Checker_Testsuite is
   use Types;
   use Ocarina.Checker;

   procedure Perform_All_Checking
     (Distributed_Application_Root : Node_Id;
      Options                      : Checker_Options);

   --  Available tests

   procedure Do_Check_Mutexes (Instance_Root : Node_Id);
   --  Check whether their is no more mutexes managed by a processor
   --  than the property Max_Mutexes_Number allow it.

   procedure Do_Check_Processes (Instance_Root : Node_Id);
   --  Check whether their is no more processes managed by a processor
   --  than the property Max_Processes_Number allow it.

   procedure Do_Check_Memory_Size (Instance_Root : Node_Id);
   --  Check whether processors memory is large enough
   --  to run all their threads

   procedure Do_Check_Stack_Size (Instance_Root : Node_Id);
   --  Check whether processors accesses to a memory large enough
   --  to host its stack

   procedure Do_Check_Buses_Size (Instance_Root : Node_Id);
   --  Check whether buses are large enough in order to accept
   --  all bound connections data without stalling.

   procedure Do_Check_Connections (Instance_Root : Node_Id);
   --  Check whether the sum of all connections of all processes
   --  bound to a given processor doesn't exceed maximum defined
   --  by the processor Max_Connections_Number property.

   procedure Do_Check_Access (Instance_Root : Node_Id);
   --  Only print the number of threads acceding (via 'access')
   --  to each data instance.

end Ocarina.Checker_Testsuite;
