------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--                OCARINA.ENTITIES.COMPONENTS.SUBCOMPONENTS                 --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--               Copyright (C) 2004-2006, GET-Telecom Paris.                --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

with Ocarina.Nodes;
with Ocarina.Entities.Components;

package body Ocarina.Entities.Components.Subcomponents is

   ----------------------------------
   -- Get_Category_Of_Subcomponent --
   ----------------------------------

   function Get_Category_Of_Subcomponent
     (Subcomponent : Types.Node_Id)
     return Component_Category
   is
      use Types;
      use Ocarina.Nodes;
      use Ocarina.Entities.Components;

      pragma Assert
        (Subcomponent /= No_Node
         and then (Kind (Subcomponent) = K_Subcomponent
                   or else Kind (Subcomponent) = K_Subcomponent_Instance));
   begin
      if Kind (Subcomponent) = K_Subcomponent then
         return Component_Category'Val (Category (Subcomponent));
      else
         if Corresponding_Declaration (Subcomponent) /= No_Node then
            return Component_Category'Val
              (Category (Corresponding_Declaration (Subcomponent)));
         else
            return CC_Unknown;
         end if;
      end if;
   end Get_Category_Of_Subcomponent;

   ---------------------------------
   -- Get_Corresponding_Component --
   ---------------------------------

   function Get_Corresponding_Component
     (Subcomponent : Types.Node_Id)
     return Types.Node_Id
   is
      use Types;
      use Ocarina.Nodes;
      use Ocarina.Entities;

      pragma Assert (Subcomponent /= No_Node
                     and then Kind (Subcomponent) = K_Subcomponent);
   begin
      if Entity_Ref (Subcomponent) /= No_Node then
         return Get_Referenced_Entity (Entity_Ref (Subcomponent));
      else
         return No_Node;
      end if;
   end Get_Corresponding_Component;

end Ocarina.Entities.Components.Subcomponents;
