------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--          O C A R I N A . E N T I T I E S . P R O P E R T I E S           --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--               Copyright (C) 2004-2007, GET-Telecom Paris.                --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

--  This package provides functions to create or read property names,
--  types, constants and associations.

with Types;
with Ocarina.AADL_Values;

package Ocarina.Entities.Properties is

   type Property_Type is
     (PT_Boolean,
      PT_Boolean_Expression,
      PT_Integer,
      PT_Unsigned_Integer,
      PT_Float,
      PT_Unsigned_Float,
      PT_String,
      PT_Enumeration,
      PT_Reference,
      PT_Classifier,
      PT_Range,
      PT_List,
      PT_Other);

   type Property_Owner_Category is
     (POC_Component_Category,
      POC_Mode,
      POC_Port_Group,
      POC_Flow,

      POC_Port,
      POC_Event_Port,
      POC_Data_Port,
      POC_Event_Data_Port,

      POC_Server_Subprogram,

      POC_Parameter,

      POC_Connections,
      POC_Port_Group_Connections,
      POC_Port_Connections,
      POC_Event_Port_Connections,
      POC_Data_Port_Connections,
      POC_Event_Data_Port_Connections,

      POC_Access_Connections,

      POC_Parameter_Connections);

   type Referable_Element_Category is
     (REC_Component_Category, REC_Connections, REC_Server_Subprogram);

   --
   --  Interrogation functions
   --

   function Value_Of_Property_Association_Is_Undefined
     (Property : Types.Node_Id)
     return Boolean;

   function Type_Of_Property_Is_A_List
     (Property : Types.Node_Id)
     return Boolean;

   function Get_Type_Of_Property
     (Property : Types.Node_Id;
      Use_Evaluated_Values : Boolean := True)
     return Property_Type;

   function Get_Type_Of_Property_Value
     (Property_Value : Types.Node_Id;
      Use_Evaluated_Values : Boolean := True)
     return Property_Type;

   function Get_Integer_Of_Property_Value
     (Property_Value : Types.Node_Id)
     return Types.Unsigned_Long_Long;

   function Get_Float_Of_Property_Value
     (Property_Value : Types.Node_Id)
     return Long_Long_Float;

   function Get_String_Of_Property_Value
     (Property_Value : Types.Node_Id)
     return Types.Name_Id;

   function Get_String_Of_Property_Value
     (Property_Value : Types.Node_Id)
     return String;

   function Get_Enumeration_Of_Property_Value
     (Property_Value : Types.Node_Id)
     return Types.Name_Id;

   function Get_Enumeration_Of_Property_Value
     (Property_Value : Types.Node_Id)
     return String;

   function Get_Boolean_Of_Property_Value
     (Property_Value : Types.Node_Id)
     return Boolean;

   function Get_Classifier_Of_Property_Value
     (Property_Value : Types.Node_Id)
     return Types.Node_Id;

   function Get_Reference_Of_Property_Value
     (Property_Value : Types.Node_Id)
     return Types.Node_Id;

   function Get_Value_Of_Property_Association
     (Property : Types.Node_Id)
     return Ocarina.AADL_Values.Value_Type;

   function Find_Property_Association_From_Name
     (Property_List : Types.List_Id;
      Property_Name : Types.Name_Id)
     return Types.Node_Id;

   function Find_Property_Association_From_Name
     (Property_List : Types.List_Id;
      Property_Name : String)
     return Types.Node_Id;

end Ocarina.Entities.Properties;
