------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--                     O C A R I N A . E N T I T I E S                      --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--               Copyright (C) 2005-2007, GET-Telecom Paris.                --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

with Types;

package Ocarina.Entities is

   type Entity_Category is
     (EC_Feature,
      EC_Subcomponent,
      EC_Property_Association,
      EC_Flow_Spec,
      EC_Flow_Implementation,
      EC_Call_Sequence,
      EC_Subprogram_Call,
      EC_Connection,
      EC_Mode,
      EC_Component,
      EC_Port_Group_Type,
      EC_Package,
      EC_Property_Set,
      EC_Property_Name,
      EC_Property_Type,
      EC_Annex,
      EC_Undefined);

   function Get_Entity_Category (Node : Types.Node_Id) return Entity_Category;

   function Get_Name_Of_Entity
     (Entity : Types.Node_Id;
      Get_Display_Name : Boolean := True)
     return Types.Name_Id;

   function Get_Name_Of_Entity
     (Entity : Types.Node_Id;
      Get_Display_Name : Boolean := True)
     return String;

   function Get_Name_Of_Entity_Reference
     (Entity_Ref : Types.Node_Id;
      Get_Display_Name : Boolean := True)
     return Types.Name_Id;

   function Get_Name_Of_Entity_Reference
     (Entity_Ref : Types.Node_Id;
      Get_Display_Name : Boolean := True)
     return String;

   function Get_Referenced_Entity
     (Entity_Ref : Types.Node_Id)
     return Types.Node_Id;
   --  Return the entity referenced by an entity reference, or No_Node
   --  if nothing is pointed

   procedure Set_Referenced_Entity (Entity_Ref, Entity : Types.Node_Id);
   --  Set the entity that is to be referenced by the entity reference

   procedure Add_Path_Element_To_Entity_Reference
     (Entity_Ref, Item : Types.Node_Id);
   --  Add Item to the end of the path that constitutes the reference
   --  to the entity.

   function Entity_Reference_Path_Has_Several_Elements
     (Entity_Ref : Types.Node_Id)
     return Boolean;
   --  return True if the path has more than one element.

   function Duplicate_Identifier
     (Identifier : Types.Node_Id)
     return Types.Node_Id;

end Ocarina.Entities;
