------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--                      O C A R I N A . P R I N T E R                       --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--               Copyright (C) 2005-2007, GET-Telecom Paris.                --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

with GNAT.OS_Lib;
with Types;

package Ocarina.Printer is

   type Output_Options is record
      Printer_Name                    : Types.Name_Id;
      Print_Standard_Property_Sets    : Boolean;
      Print_Evaluated_Property_Values : Boolean;
      Output_File                     : Types.Name_Id;

      Output_Directory                : Types.Name_Id;
      --  This field is not used for now. It's kept for future use
      --  when multi-file printing is implemented
   end record;

   Default_Output_Options : constant Output_Options :=
     (Printer_Name                    => Types.No_Name,
      Print_Standard_Property_Sets    => False,
      Print_Evaluated_Property_Values => False,
      Output_File                     => Types.No_Name,
      Output_Directory                => Types.No_Name);

   type Printing_Facility_Access is access procedure
     (Node    : Types.Node_Id;
      Options : Output_Options := Default_Output_Options);

   procedure Register_Printer
     (Module  : String;
      Printer : Printing_Facility_Access);

   procedure Reset_Printers;
   --  Unregister all the registered printers

   function Create_Output_File
     (Options : Output_Options)
     return GNAT.OS_Lib.File_Descriptor;
   --  Returns a file descriptor corresponding to the print output.
   --  if no file name is given, printing will be done in the standard
   --  output

   function Print
     (Root    : Types.Node_Id;
      Options : Output_Options)
     return Boolean;

   procedure Write_Printers (Indent : Natural);
   --  List all the registered printers

end Ocarina.Printer;
