------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--           O C A R I N A . V I S I T O R . C O M P O N E N T S            --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--               Copyright (C) 2005-2006, GET-Telecom Paris.                --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

with Ocarina.Nodes;

package body Ocarina.Visitor.Components is

   function Visit_All_Components
     (Root : Types.Node_Id;
      Callback : Component_Callback)
     return Boolean
   is
      use Types;
      use Ocarina.Nodes;

      pragma Assert (Root /= No_Node
                     and then Kind (Root) = K_AADL_Specification);

      Success : Boolean := True;
      List_Node, Package_List_Node : Node_Id;
   begin
      if Declarations (Root) /= No_List then
         List_Node := First_Node (Declarations (Root));

         while List_Node /= No_Node loop
            case Kind (List_Node) is
               when  K_Component_Implementation
                 | K_Component_Type
                 | K_Port_Group_Type =>
                  Success := Callback (Root => Root,
                                       Container => Root,
                                       Component => List_Node)
                    and then Success;

               when K_Package_Specification =>
                  if Declarations (List_Node) /= No_List then
                     Package_List_Node :=
                       First_Node (Declarations (List_Node));

                     while Package_List_Node /= No_Node loop
                        case Kind (Package_List_Node) is
                           when  K_Component_Implementation
                             | K_Component_Type
                             | K_Port_Group_Type =>
                              Success := Callback
                                (Root => Root,
                                 Container => List_Node,
                                 Component => Package_List_Node)
                                and then Success;

                           when others =>
                              null;
                        end case;

                        Package_List_Node := Next_Node (Package_List_Node);
                     end loop;
                  end if;

               when others =>
                  null;
            end case;

            List_Node := Next_Node (List_Node);
         end loop;
      end if;

      return Success;
   end Visit_All_Components;

end Ocarina.Visitor.Components;
