------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--                 OCARINA.DIA.PARSER.TRANSLATE.COMPONENTS                  --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--               Copyright (C) 2005-2006, GET-Telecom Paris.                --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

with Types;
with Locations;

with Ocarina.Entities.Components;
with Ocarina.Dia.Parser.Core;
use Ocarina.Dia.Parser.Core;

package Ocarina.Dia.Parser.Translate.Components is

   L : constant Locations.Location := Locations.No_Location;

   -----------------------
   --  Create_Component --
   -----------------------

   function Create_Component
      (Dia_Node : Ocarina.Dia.Parser.Core.Node;
       Ocarina_Root : Types.Node_Id;
       Category : Ocarina.Entities.Components.Component_Category;
       NL : Node_List)
      return Types.Node_Id;

   ------------------------------
   --  Add_New_Component_Type  --
   ------------------------------

   function Add_New_Component_Type
      (Identifier : Types.Node_Id;
       Namespace : Types.Node_Id;
       Component_Type : Ocarina.Entities.Components.Component_Category)
      return Types.Node_Id;
   --  todo What to do with the Is_Private ?
   --  Create a new component type node. Identifier is a Node_Id
   --  referencing the name of the component. Namespace is either a
   --  package specification or the top level AADL specification.
   --  Component_Type is the component category
   --  (processor, memory, process, etc.) Is_Private indicates if
   --  the component is declaraed in the private or the public part
   --  of the package; it is only relevant if Namespace references a
   --  package specification. Returns the Node_Id of the newly created
   --  component type node, or No_Node if something went wrong.

   function Add_New_Component_Implementation
      (Identifier : Types.Node_Id;
       Namespace : Types.Node_Id;
       Component_Type : Ocarina.Entities.Components.Component_Category)
      return Types.Node_Id;
   --  todo What to do with the Is_Private ?
   --  Create a new component implementation node. Identifier is a
   --  Node_Id referencing the name of the component. Namespace is
   --  either a package specification or the top level AADL
   --  specification.Component_Type is the component category
   --  (processor, memory, process, etc.) Is_Private indicates if
   --  the component is declaraed in the private or the public part
   --  of the package; it is only relevant if Namespace references a
   --  package specification. Returns the Node_Id of the newly created
   --  component type node, or No_Node if something went wrong.

   function Is_Implementation (N : Node; List : Node_List)
      return Boolean;

   function Get_Impl_Source_Identifier (DN : Node)
      return Types.Node_Id;

end Ocarina.Dia.Parser.Translate.Components;
