------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--          O C A R I N A . D I A . P R I N T E R . O B J E C T S           --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--               Copyright (C) 2005-2006, GET-Telecom Paris.                --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

--  This package contains the top-level procedures to create
--  the objects of the Dia file

package Ocarina.Dia.Printer.Objects is

   --  adds the attribute name in an object
   procedure Add_Name
     (Doc : DOM.Core.Document;
      Parent_Node : DOM.Core.Node;
      Name : String);

   --  adds an object to the XML Tree
   function Add_Object
     (Doc : DOM.Core.Document;
      Parent_Node : DOM.Core.Node;
      Node : Node_Id;
      Id : String;
      Parent_String : String;
      Typ : String;
      Decl : String;
      Name : String)
     return DOM.Core.Node;

   --  returns the name of an AADL Object
   function Get_Name (Node : Node_Id) return String;

   --  returns the name of an XML Object
   function Get_Name_XML_Object (XML_Node : DOM.Core.Node) return String;

   --  return the XML node aadl_Box_Ports of an xml object
   function Get_Aadlbox_Ports
     (XML_Object : DOM.Core.Node)
     return DOM.Core.Node;

   --  extracts the features from an Ocarina Object
   procedure Create_Feature
     (Doc : DOM.Core.Document;
      Parent_Node : DOM.Core.Node;
      Node : Node_Id);

   --  extracts the ports from an Ocarina Node
   procedure Create_Ports
     (Doc : DOM.Core.Document;
      Parent_Node : DOM.Core.Node;
      Node : Node_Id);

   --  returns the Category of the Object to be used in the dia file
   function Node_To_String (Node : Node_Id) return String;

   --  this function will create the appropriate Object in the DOM
   --  Tree
   function Create_Component_Type
     (Doc : DOM.Core.Document;
      Parent_Node : DOM.Core.Node;
      Node : Node_Id;
      Id : Integer;
      Parent : Integer)
     return Integer;

   --  this function Create a component implementation in the dia
   --  file.
   function Create_Component_Implementation
     (Doc : DOM.Core.Document;
      Parent_Node : DOM.Core.Node;
      Node : Node_Id;
      Id : Integer;
      Parent : Integer)
     return Integer;

   --  this function Create a subcomponent
   function Create_Subcomponent
     (Doc : DOM.Core.Document;
      Parent_Node : DOM.Core.Node;
      Node : Node_Id;
      Id : Integer;
      Id_Parent : Integer)
     return Integer;

   --  this function Create a call
   function Create_Call
     (Doc : DOM.Core.Document;
      Parent_Node : DOM.Core.Node;
      Node : Node_Id;
      Id : Integer)
     return Integer;

   --  this function Create a flow
   function Create_Flow
     (Doc : DOM.Core.Document;
      Parent_Node : DOM.Core.Node;
      Node : Node_Id;
      Id : Integer)
     return Integer;

   --  this function Create a mode
   function Create_Mode
      (Doc : DOM.Core.Document;
       Parent_Node : DOM.Core.Node;
       Node : Node_Id;
       Id : Integer)
      return Integer;

   --  this function Create a package
   function Create_Package
     (Doc : DOM.Core.Document;
      Parent_Node : DOM.Core.Node;
      Node : Node_Id;
      Id : Integer)
     return Integer;

end Ocarina.Dia.Printer.Objects;
