------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--                      G A I A . T R A N S L A T O R                       --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--               Copyright (C) 2005-2006, GET-Telecom Paris.                --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

with Namet;
with Output;

package body Gaia.Translator is

   --  The chained list type that contains the registered translator modules

   type Translator_Module;
   type Translator_Module_Access is access all Translator_Module;
   type Translator_Module is
      record
         Module_Name       : Types.Name_Id;
         Module_Subprogram : Translator_Facility_Access;
         Next_Module       : Translator_Module_Access;
      end record;

   Gaia_Translators : Translator_Module_Access := null;

   -------------------------
   -- Register_Translator --
   -------------------------

   procedure Register_Translator
     (Translator_Name : String;
      Translator      : Translator_Facility_Access)
   is
      use Namet;

      The_Translator : constant Translator_Module_Access
        := new Translator_Module;
   begin
      if Translator_Name'Length > 0 then
         Set_Str_To_Name_Buffer (Translator_Name);
         The_Translator.Module_Name := Name_Find;
         The_Translator.Module_Subprogram := Translator;
         The_Translator.Next_Module := Gaia_Translators;
         Gaia_Translators := The_Translator;
      end if;
   end Register_Translator;

   -----------------------
   -- Select_Translator --
   -----------------------

   function Select_Translator
     (Module : String)
     return Translator_Facility_Access
   is
      use Namet;
      use Types;

      Current_Translator :  Translator_Module_Access := Gaia_Translators;
      Module_Name        : Name_Id;
   begin
      if Module'Length > 0 then
         Set_Str_To_Name_Buffer (Module);
         Module_Name := Name_Find;

         while Current_Translator /= null loop
            if Current_Translator.Module_Name = Module_Name then
               return Current_Translator.Module_Subprogram;
            end if;

            Current_Translator := Current_Translator.Next_Module;
         end loop;
      end if;

      return null;
   end Select_Translator;

   -----------------------
   -- Write_Translators --
   -----------------------

   procedure Write_Translators (Indent : Natural) is
      use Output;
      use Namet;
      Current_Translator :  Translator_Module_Access := Gaia_Translators;
   begin
      for Index in 1 .. Indent loop
            Write_Char (' ');
      end loop;
      Write_Line ("Registered translators:");

      while Current_Translator /= null loop
         for Index in 1 .. Indent + 1 loop
            Write_Char (' ');
         end loop;

         Write_Name (Current_Translator.Module_Name);
         Write_Eol;

         Current_Translator := Current_Translator.Next_Module;
      end loop;
   end Write_Translators;

end Gaia.Translator;
