------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--       O C A R I N A . G E N E R A T O R S . B U I L D _ U T I L S        --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--                 Copyright (C) 2007, GET-Telecom Paris.                   --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

--  This package gathers all the necessary routines to generate
--  makefiles, Ada project files or what ever seems (in the future)
--  necessary to make the build of a generated application simple. It
--  consist of several nested packages that contain mainly a couple of
--  procedures:

--  "Visit" which is a classical visitor that travers the AADL
--  instance tree and defines the structure of the corresponding
--  entity that will be generated.

--  "Generate" which dumps the internal structure corresponding to a
--  given node into a file.

package Ocarina.Generators.Build_Utils is

   ----------------------------------
   -- Makefile generation routines --
   ----------------------------------

   package Makefiles is

      procedure Visit (E : Node_Id);
      procedure Generate (E : Node_Id);
      procedure Build (E : Node_Id);
      procedure Clean (E : Node_Id);

      procedure Reset;
      --  Deallocates the internals of this package

   end Makefiles;

   ------------------------------------------
   -- Ada project file generation routines --
   ------------------------------------------

   package Ada_Project_Files is

      procedure Visit (E : Node_Id);
      procedure Generate (E : Node_Id);

      procedure Reset;
      --  Deallocates the internals of this package

   end Ada_Project_Files;

   procedure Reset;
   --  Deallocate all the internal data

end Ocarina.Generators.Build_Utils;
