------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--                  OCARINA.GENERATORS.PO_HI_ADA.RUNTIME                    --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--               Copyright (C) 2006-2007, GET-Telecom Paris.                --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

package Ocarina.Generators.PO_HI_Ada.Runtime is

   --  Runtime Units

   type RU_Id is
     (RU_Null,
      RU_Ada,
      RU_Ada_Exceptions,
      RU_Ada_Real_Time,
      RU_Ada_Streams,
      RU_ASN1_Types,
      RU_PolyORB_HI,
      RU_PolyORB_HI_Periodic_Task,
      RU_PolyORB_HI_Sporadic_Task,
      RU_Deployment,
      RU_GNAT,
      RU_GNAT_Sockets,
      RU_Interfaces,
      RU_Interfaces_C,
      RU_PolyORB_HI_Marshallers_G,
      RU_Marshallers,
      RU_PolyORB_HI_Messages,
      RU_Naming,
      RU_PolyORB_HI_Output_Low_Level,
      RU_PolyORB_HI_Output,
      RU_PolyORB_HI_Protocols,
      RU_Standard,
      RU_Subprograms,
      RU_PolyORB_HI_Thread_Interrogators,
      RU_PolyORB_HI_Transport_Low_Level,
      RU_PolyORB_HI_Transport_Low_Level_Extras,
      RU_PolyORB_HI_Transport,
      RU_Types,
      RU_SOIF_MTS_Wrapper_Ada,
      RU_SOIF_MTS_Wrapper_Ada_Structures,
      RU_PolyORB_HI_Suspenders,
      RU_System_Time,
      RU_System);

   --  Runtime Entities

   type RE_Id is
     (RE_Null,                       --  Workaround to denote a null RE
      RE_Off,                        --  Off
      RE_On,                         --  On
      RE_True,                       --  True
      RE_False,                      --  False
      RE_SOIF_MTS_Naming_Entry,      --  SOIF_MTS_Naming_Entry
      RE_SOIF_MTS_Naming_Store,      --  SOIF_MTS_Naming_Store
      RE_Node_Type,                  --  Deployment.Node_Type
      RE_Entity_Type,                --  Deployment.Entity_Type
      RE_Port_Type_1,                --  Deployment.Port_Type
      RE_Clock,                      --  Ada.Real_Time.Clock
      RE_Nanoseconds,                --  Ada.Real_Time.Nanoseconds
      RE_Microseconds,               --  Ada.Real_Time.Microseconds
      RE_Milliseconds,               --  Ada.Real_Time.Milliseconds
      RE_Seconds,                    --  Ada.Real_Time.Seconds
      RE_Minutes,                    --  Ada.Real_Time.Minutes
      RE_Time,                       --  Ada.Real_Time.Time
      RE_Time_Span,                  --  Ada.Real_Time.Time_Span
      RE_Stream_Element_Array,       --  Ada.Streams.Stream_Element_Array
      RE_Stream_Element_Offset,      --  Ada.Streams.Stream_Element_Offset
      RE_Family_Inet,                --  GNAT.Sockets.Family_Inet
      RE_Inet_Addr,                  --  GNAT.Sockets.Inet_Addr
      RE_No_Inet_Addr,               --  GNAT.Sockets.No_Inet_Addr
      RE_No_Port,                    --  GNAT.Sockets.No_Port
      RE_Port_Type_2,                --  GNAT.Sockets.Port_Type
      RE_Sock_Addr_Type,             --  GNAT.Sockets.Sock_Addr_Type
      RE_Char_Array,                 --  Interfaces.C.char_array
      RE_Nul,                        --  Interfaces.C.nul
      RE_Marshall_Request,           --  Marshallers.Marshall_Request
      RE_Unmarshall_Request,         --  Marshallers.Unmarshall_Request
      RE_Write,                      --  Messages.Write
      RE_Header_Size,                --  Messages.Header_Size
      RE_Message_Type,               --  Messages.Message_Type,
      RE_Reallocate,                 --  Messages.Reallocate
      RE_Non_Blocking_Receive,       --  Protocols.Non_Blocking_Receive
      RE_Send,                       --  Protocols.Send
      RE_Initialize,                 --  Transport.Initialize
      RE_SOIF_MTS_ASSERT_V1_SPACEWIRE_PID, --  SOIF_MTS_Wrapper_Ada
      RE_SOIF_MTS_NON_GUARANTEED_DELIVERY, --  SOIF_MTS_Wrapper_Ada
      RE_SOIF_MTS_Naming_Entry_Type, --  SOIF_MTS_Wrapper_Ada.Structures
      --                                   .SOIF_MTS_Naming_Entry_Type
      RE_SOIF_MTS_Naming_Store_Type, --  SOIF_MTS_Wrapper_Ada.Structures
      --                                   .SOIF_MTS_Naming_Store_Type
      RE_Boolean,                    --  Standard.Boolean
      RE_Integer,                    --  Standard.Integer
      RE_Natural,                    --  Standard.Natural
      RE_Positive,                   --  Standard.Positive
      RE_Float,                      --  Standard.Float
      RE_String,                     --  Standard.String
      RE_Character,                  --  Standard.Character
      RE_Wide_Character,             --  Standard.Wide_Character
      RE_Suspend_Forever,            --  Suspenders.Suspend_Forever
      RE_Unblock_All_Tasks,          --  Suspenders.Unblock_All_Tasks
      RE_Address,                    --  System.Address
      RE_Null_Address,               --  System.Null_Address
      RE_Default_Priority,           --  System.Default_Priority
      RE_Priority);                  --  System.Priority

   RE_Unit_Table : constant array (RE_Id) of RU_Id
     := (RE_Null                       => RU_Null,
         RE_Off                        => RU_Null,
         RE_On                         => RU_Null,
         RE_True                       => RU_Null,
         RE_False                      => RU_Null,
         RE_SOIF_MTS_Naming_Entry      => RU_Null,
         RE_SOIF_MTS_Naming_Store      => RU_Null,
         RE_Node_Type                  => RU_Deployment,
         RE_Entity_Type                => RU_Deployment,
         RE_Port_Type_1                => RU_Deployment,
         RE_Clock                      => RU_Ada_Real_Time,
         RE_Nanoseconds                => RU_Ada_Real_Time,
         RE_Microseconds               => RU_Ada_Real_Time,
         RE_Milliseconds               => RU_Ada_Real_Time,
         RE_Seconds                    => RU_Ada_Real_Time,
         RE_Minutes                    => RU_Ada_Real_Time,
         RE_Time                       => RU_Ada_Real_Time,
         RE_Time_Span                  => RU_Ada_Real_Time,
         RE_Stream_Element_Array       => RU_Ada_Streams,
         RE_Stream_Element_Offset      => RU_Ada_Streams,
         RE_Family_Inet                => RU_GNAT_Sockets,
         RE_Inet_Addr                  => RU_GNAT_Sockets,
         RE_No_Inet_Addr               => RU_GNAT_Sockets,
         RE_No_Port                    => RU_GNAT_Sockets,
         RE_Port_Type_2                => RU_GNAT_Sockets,
         RE_Sock_Addr_Type             => RU_GNAT_Sockets,
         RE_Char_Array                 => RU_Interfaces_C,
         RE_Nul                        => RU_Interfaces_C,
         RE_Marshall_Request           => RU_Marshallers,
         RE_Unmarshall_Request         => RU_Marshallers,
         RE_Write                      => RU_PolyORB_HI_Messages,
         RE_Header_Size                => RU_PolyORB_HI_Messages,
         RE_Message_Type               => RU_PolyORB_HI_Messages,
         RE_Reallocate                 => RU_PolyORB_HI_Messages,
         RE_Non_Blocking_Receive       => RU_PolyORB_HI_Protocols,
         RE_Send                       => RU_PolyORB_HI_Protocols,
         RE_Initialize                 => RU_PolyORB_HI_Transport,
         RE_SOIF_MTS_ASSERT_V1_SPACEWIRE_PID
           => RU_SOIF_MTS_Wrapper_Ada_Structures,
         RE_SOIF_MTS_NON_GUARANTEED_DELIVERY
           => RU_SOIF_MTS_Wrapper_Ada_Structures,
         RE_SOIF_MTS_Naming_Entry_Type => RU_SOIF_MTS_Wrapper_Ada_Structures,
         RE_SOIF_MTS_Naming_Store_Type => RU_SOIF_MTS_Wrapper_Ada_Structures,
         RE_Boolean                    => RU_Standard,
         RE_Integer                    => RU_Standard,
         RE_Natural                    => RU_Standard,
         RE_Positive                   => RU_Standard,
         RE_Float                      => RU_Standard,
         RE_String                     => RU_Standard,
         RE_Character                  => RU_Standard,
         RE_Wide_Character             => RU_Standard,
         RE_Suspend_Forever            => RU_PolyORB_HI_Suspenders,
         RE_Unblock_All_Tasks          => RU_PolyORB_HI_Suspenders,
         RE_Address                    => RU_System,
         RE_Null_Address               => RU_System,
         RE_Default_Priority           => RU_System,
         RE_Priority                   => RU_System);

   procedure Initialize;
   procedure Reset;

   function RE (Id : RE_Id; Withed : Boolean := True) return Node_Id;
   --  Return a designator for entity Id

   function RU (Id : RU_Id; Withed : Boolean := True) return Node_Id;
   --  Return a node for Unit Id

end Ocarina.Generators.PO_HI_Ada.Runtime;
