------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--               OCARINA.GENERATORS.PO_QOS_ADA.OBJ_ADAPTERS                 --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--               Copyright (C) 2006-2007, GET-Telecom Paris.                --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

with Ocarina.Nodes;
with Ocarina.Nutils;
with Ocarina.Entities.Components;

with Ocarina.Generators.Utils;
with Ocarina.Generators.Ada_Tree.Nutils;
with Ocarina.Generators.Ada_Tree.Nodes;
with Ocarina.Generators.PO_QoS_Ada.Runtime;
with Ocarina.Generators.Ada_Values;

package body Ocarina.Generators.PO_QoS_Ada.Obj_Adapters is

   use Ocarina.Nodes;
   use Ocarina.Entities.Components;
   use Ocarina.Generators.Utils;
   use Ocarina.Generators.Ada_Tree.Nutils;
   use Ocarina.Generators.PO_QoS_Ada.Runtime;
   use Ocarina.Generators.Ada_Values;

   package AAN renames Ocarina.Nodes;
   package AAU renames Ocarina.Nutils;
   package ADN renames Ocarina.Generators.Ada_Tree.Nodes;

   pragma Unreferenced (AAN);

   ------------------
   -- Package_Spec --
   ------------------

   package body Package_Spec is

      procedure Visit_Architecture_Instance (E : Node_Id);
      procedure Visit_Component_Instance (E : Node_Id);
      procedure Visit_System_Instance (E : Node_Id);
      procedure Visit_Process_Instance (E : Node_Id);

      -----------
      -- Visit --
      -----------

      procedure Visit (E : Node_Id) is
      begin
         case Kind (E) is
            when K_Architecture_Instance =>
               Visit_Architecture_Instance (E);

            when K_Component_Instance =>
               Visit_Component_Instance (E);

            when others =>
               null;
         end case;
      end Visit;

      ---------------------------------
      -- Visit_Architecture_Instance --
      ---------------------------------

      procedure Visit_Architecture_Instance (E : Node_Id) is
      begin
         Visit (Root_System (E));
      end Visit_Architecture_Instance;

      ------------------------------
      -- Visit_Component_Instance --
      ------------------------------

      procedure Visit_Component_Instance (E : Node_Id) is
         Cathegory : constant Component_Category
           := Get_Category_Of_Component (E);
      begin
         case Cathegory is
            when CC_System =>
               Visit_System_Instance (E);

            when CC_Process =>
               Visit_Process_Instance (E);

            when others =>
               null;
         end case;
      end Visit_Component_Instance;

      ----------------------------
      -- Visit_Process_Instance --
      ----------------------------

      procedure Visit_Process_Instance (E : Node_Id) is
         U          : constant Node_Id := ADN.Distributed_Application_Unit
           (ADN.Helpers_Node (Backend_Node (Identifier (E))));
         P          : constant Node_Id := ADN.Entity (U);
         N          : Node_Id;
         S          : Node_Id;
         N_Servants : Unsigned_Long_Long := 0;
      begin
         Push_Entity (P);
         Push_Entity (U);

         Set_Obj_Adapters_Spec;

         --  Count the number of threads having IN ports (servants)

         if not AAU.Is_Empty (Subcomponents (E)) then
            S := First_Node (Subcomponents (E));

            while Present (S) loop
               --  Visit the corresponding component instance

               if Is_Thread (Corresponding_Instance (S)) and then
                 Has_In_Ports (Corresponding_Instance (S))
               then
                  N_Servants := N_Servants + 1;
               end if;

               S := Next_Node (S);
            end loop;
         end if;

         --  Do not generate the Object_Adapter package instantiation
         --  only if there are servants in on the node.

         if N_Servants > 0 then
            N := Make_Literal (New_Integer_Value (N_Servants, 1, 10));

            --  Instantiate the generic package

            N := Make_Package_Instantiation
              (Defining_Identifier => ADN.Defining_Identifier
                 (ADN.Package_Declaration
                  (Current_Package)),
               Generic_Package     => RU (RU_ARAO_Object_Adapter_G),
               Parameter_List      => Make_List_Id (N));

            ADN.Set_Is_Instantiated_Package (Current_Package, True);
            ADN.Set_Package_Instantiation (Current_Package, N);
         end if;

         Pop_Entity; --  U
         Pop_Entity; --  P
      end Visit_Process_Instance;

      ---------------------------
      -- Visit_System_Instance --
      ---------------------------

      procedure Visit_System_Instance (E : Node_Id) is
         S : Node_Id;
      begin
         Push_Entity (QoS_Distributed_Application_Root);

         if not AAU.Is_Empty (Subcomponents (E)) then
            S := First_Node (Subcomponents (E));

            while Present (S) loop
               --  Visit the corresponding component instance

               Visit (Corresponding_Instance (S));

               S := Next_Node (S);
            end loop;
         end if;

         Pop_Entity; --  QoS_Distributed_Application_Root
      end Visit_System_Instance;

   end Package_Spec;

end Ocarina.Generators.PO_QoS_Ada.Obj_Adapters;
