------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--                      O C A R I N A . P N . R O O T                       --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--                 Copyright (C) 2006, GET-Telecom Paris.                   --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

with Namet;
with Ocarina.PN.Nodes;
with Ocarina.PN.Nutils;

package body Ocarina.PN.Root is

   ----------------------------
   -- Create_PN_Declarations --
   ----------------------------

   procedure Create_PN_Declarations (PN_Root : Types.Node_Id) is
      use Types;
      use Namet;
      use Ocarina.PN.Nutils;
      use Ocarina.PN.Nodes;
      use Ocarina.AADL_Values;

      Decl, Val : Node_Id;
   begin
      --  Value

      Decl := Class_Value (PN_Root);
      Set_Symbolic_Values (Decl, New_List (K_List_Id));
      Val := New_Node (K_Color_Variable);
      Set_Str_To_Name_Buffer ("u");
      Set_Value (Val, New_String_Value (Name_Find));
      Append_Node_To_List (Val, Symbolic_Values (Decl));
      Val := New_Node (K_Color_Variable);
      Set_Str_To_Name_Buffer ("d");
      Set_Value (Val, New_String_Value (Name_Find));
      Append_Node_To_List (Val, Symbolic_Values (Decl));
      Append_Node_To_List (Decl, Color_Declarations (PN_Root));

      --  Control

      Decl := Class_Control (PN_Root);
      Append_Node_To_List (Decl, Color_Declarations (PN_Root));

      --  Comm

      Decl := Domain_Comm (PN_Root);
      Set_Classes (Decl, New_List (K_List_Id));
      Val := New_Node (K_Color_Variable);
      Set_Str_To_Name_Buffer ("Value");
      Set_Value (Val, New_String_Value (Name_Find));
      Append_Node_To_List (Val, Classes (Decl));
      Val := New_Node (K_Color_Variable);
      Set_Str_To_Name_Buffer ("Control");
      Set_Value (Val, New_String_Value (Name_Find));
      Append_Node_To_List (Val, Classes (Decl));
      Append_Node_To_List (Decl, Color_Declarations (PN_Root));
   end Create_PN_Declarations;

   ------------------------
   -- Declare_New_Thread --
   ------------------------

   function Declare_New_Thread
     (PN_Root : Types.Node_Id)
     return Ocarina.AADL_Values.Value_Type
   is
      use Types;
      use Ocarina.AADL_Values;
      use Ocarina.PN.Nodes;

      Nb : Value_Type;
   begin
      Nb := Get_Value_Type (Number_Of_Threads (PN_Root));
      Nb.IVal := Nb.IVal + 1;
      Set_Value (Number_Of_Threads (PN_Root), Nb);
      return Nb;
   end Declare_New_Thread;

   ------------------------------
   -- Get_New_Control_Variable --
   ------------------------------

   function Get_New_Control_Variable
     (PN_Root, Current_Variable : Types.Node_Id)
     return Types.Node_Id
   is
      use Types;
      use Namet;
      use Ocarina.PN.Nodes;
      use Ocarina.AADL_Values;
      use Ocarina.PN.Nutils;

      pragma Assert (PN_Root /= No_Node
                       and then Kind (PN_Root) = K_Root_Node);
      pragma Assert (Current_Variable = No_Node
                       or else Kind (Current_Variable) = K_Color_Variable);

      New_Var : Node_Id;
      Nb_Of_Vars : Value_Type;
   begin
      if Current_Variable = No_Node then
         New_Var := First_Node (Variables (Control_Variables (PN_Root)));
      else
         New_Var := Next_Node (Current_Variable);
      end if;

      if New_Var = No_Node then
         Nb_Of_Vars := Get_Value_Type (Number_Of_Control_Vars (PN_Root));
         Nb_Of_Vars.IVal := Nb_Of_Vars.IVal + 1;
         Set_Value (Number_Of_Control_Vars (PN_Root), Nb_Of_Vars);
         New_Var := New_Node (K_Color_Variable);

         declare
            S : constant String := Unsigned_Long_Long'Image (Nb_Of_Vars.IVal);
         begin
            Set_Str_To_Name_Buffer ("c" & S (S'First + 1 .. S'Last));
         end;

         Set_Value (New_Var, New_String_Value (Name_Find));
         Append_Node_To_List (New_Var,
                              Variables (Control_Variables (PN_Root)));
      end if;

      return New_Var;
   end Get_New_Control_Variable;

   ----------------------------
   -- Get_New_Value_Variable --
   ----------------------------

   function Get_New_Value_Variable
     (PN_Root, Current_Variable : Types.Node_Id)
     return Types.Node_Id
   is
      use Types;
      use Namet;
      use Ocarina.PN.Nodes;
      use Ocarina.AADL_Values;
      use Ocarina.PN.Nutils;

      pragma Assert (PN_Root /= No_Node
                       and then Kind (PN_Root) = K_Root_Node);
      pragma Assert (Current_Variable = No_Node
                       or else Kind (Current_Variable) = K_Color_Variable);

      New_Var : Node_Id;
      Nb_Of_Vars : Value_Type;
   begin
      if Current_Variable = No_Node then
         New_Var := First_Node (Variables (Value_Variables (PN_Root)));
      else
         New_Var := Next_Node (Current_Variable);
      end if;

      if New_Var = No_Node then
         Nb_Of_Vars := Get_Value_Type (Number_Of_Value_Vars (PN_Root));
         Nb_Of_Vars.IVal := Nb_Of_Vars.IVal + 1;
         Set_Value (Number_Of_Value_Vars (PN_Root), Nb_Of_Vars);
         New_Var := New_Node (K_Color_Variable);

         declare
            S : constant String := Unsigned_Long_Long'Image (Nb_Of_Vars.IVal);
         begin
            Set_Str_To_Name_Buffer ("v" & S (S'First + 1 .. S'Last));
         end;

         Set_Value (New_Var, New_String_Value (Name_Find));
         Append_Node_To_List (New_Var,
                              Variables (Value_Variables (PN_Root)));
      end if;

      return New_Var;
   end Get_New_Value_Variable;

   --------------------
   -- Make_Root_Node --
   --------------------

   function Make_Root_Node return Types.Node_Id is
      use Types;
      use Namet;
      use Ocarina.AADL_Values;
      use Ocarina.PN.Nutils;
      use Ocarina.PN.Nodes;

      PN_Root : constant Node_Id := New_Node (K_Root_Node);
      Decl, Ident : Node_Id;
   begin
      Set_Places (PN_Root, New_List (K_List_Id));
      Set_Arcs (PN_Root, New_List (K_List_Id));
      Set_Connections (PN_Root, New_List (K_List_Id));
      Set_Place_Fusions (PN_Root, New_List (K_List_Id));
      Set_Arc_Fusions (PN_Root, New_List (K_List_Id));
      Set_Subnets (PN_Root, New_List (K_List_Id));
      Set_Color_Declarations (PN_Root, New_List (K_List_Id));

      Set_Number_Of_Value_Vars (PN_Root, New_Integer_Value (0));
      Set_Number_Of_Control_Vars (PN_Root, New_Integer_Value (0));
      Set_Number_Of_Threads (PN_Root, New_Integer_Value (0));

      --  Value

      Decl := New_Node (K_Enumeration_Class_Declaration);
      Set_Str_To_Name_Buffer ("Value");
      Ident := Make_Identifier (Decl, Name_Find);
      Set_Identifier (Decl, Ident);
      Set_Class_Value (PN_Root, Decl);

      --  Control

      Decl := New_Node (K_Range_Class_Declaration);
      Set_Str_To_Name_Buffer ("Control");
      Ident := Make_Identifier (Decl, Name_Find);
      Set_Identifier (Decl, Ident);
      Set_Lower_Value (Decl, New_Integer_Value (1));
      Set_Higher_Value (Decl, Number_Of_Threads (PN_Root));
      Set_Class_Control (PN_Root, Decl);

      --  Comm

      Decl := New_Node (K_Domain_Declaration);
      Set_Str_To_Name_Buffer ("Comm");
      Ident := Make_Identifier (Decl, Name_Find);
      Set_Identifier (Decl, Ident);
      Set_Domain_Comm (PN_Root, Decl);

      --  Variables

      Set_Value_Variables (PN_Root, New_Node (K_Variables_Declaration));
      Set_Variables (Value_Variables (PN_Root), New_List (K_List_Id));
      Set_Identifier (Value_Variables (PN_Root),
                      Make_Identifier (Value_Variables (PN_Root),
                                       Name (Identifier
                                             (Class_Value (PN_Root)))));
      Append_Node_To_List (Value_Variables (PN_Root),
                           Color_Declarations (PN_Root));

      Set_Control_Variables (PN_Root, New_Node (K_Variables_Declaration));
      Set_Variables (Control_Variables (PN_Root), New_List (K_List_Id));
      Set_Identifier (Control_Variables (PN_Root),
                      Make_Identifier (Control_Variables (PN_Root),
                                       Name (Identifier
                                             (Class_Control (PN_Root)))));
      Append_Node_To_List (Control_Variables (PN_Root),
                           Color_Declarations (PN_Root));

      return PN_Root;
   end Make_Root_Node;

end Ocarina.PN.Root;
